using System.Collections.Generic;

namespace MediatorDesignDemo
{
    public class ChatMediator
    {
        private IList<ChatClient> _chatClients = new List<ChatClient>();

        public ChatMediator(ChatClient chatClient1, ChatClient chatClient2)
        {
            SetupChatClient(chatClient1);
            SetupChatClient(chatClient2);
        }

        public void JoinConversation(ChatClient clientJoiningConversation)
        {
            SetupChatClient(clientJoiningConversation);
        }

        public void LeaveConversation(ChatClient clientLeavingConversation)
        {
            RemoveClientFromConversation(clientLeavingConversation);
        }

        private void RemoveClientFromConversation(ChatClient clientLeavingConversation)
        {
            if (_chatClients.Contains(clientLeavingConversation))
            {
                clientLeavingConversation.SendMessageNotification -= HandleMessageSentNotifaction;
                _chatClients.Remove(clientLeavingConversation);
            }
        }

        private void SetupChatClient(ChatClient client)
        {
            client.SendMessageNotification += HandleMessageSentNotifaction;

            _chatClients.Add(client);
        }

        private void HandleMessageSentNotifaction(object sender, string message)
        {
            var clientWhoSentMessage = (ChatClient)sender;
            foreach (var chatClient in _chatClients)
            {
                if (!chatClient.Equals(clientWhoSentMessage))
                {
                    chatClient.AcceptNewMessage(clientWhoSentMessage, message);
                }
            }
        }        
    }
}